*ORIGINAL DATA FROM http://www.ats.ucla.edu/stat/sas/library/repeated_ut.htm;
OPTIONS NONUMBER NODATE NOCENTER PAGESIZE=MAX LINESIZE=120 formdlim='-';
* Kill default titles on output;
TITLE;

ODS LISTING;
ODS HTML CLOSE;

******************************************************************************;
***** Make MVN data;
******************************************************************************;
%MACRO mvn(varcov=,       /* dataset for variance-covariance matrix */
           means=,        /* dataset for mean vector */
           n=,             /* sample size */
           seed=,        /* seed for random number generator */
           sample= );     /* output dataset name */
* Get initial seed value. If seed<=0, then generate seed from system clock;
data _null_;
   if &seed. le 0 then do;
      seed = int(time());  		  /* get clock time in integer seconds */
      put seed=;
      call symput('seed',seed);   /* store seed as macro variable */
   end;
run;

* Generate the multivariate normal data in SAS/IML;
proc iml worksize=100;
   use &varcov.;            	/* read variance-covariance matrix */
   read all into cov;
   use &means.;             	/* read means */
   read all into mu;
   v=nrow(cov);            		/* calculate number of variables */
   n=&n.;
   seed = &seed;
   l=t(root(cov));         		/* calculate cholesky root of cov matrix */
   z=normal(j(v,&n.,&seed.));	/* generate nvars*samplesize normals */
   x=l*z;                  		/* premultiply by cholesky root */
   x=repeat(mu,1,&n.)+x;    	/* add in the means */
   tx=t(x);
   create &sample. from tx;  	/* write out sample data to sas dataset */
   append from tx;
quit;
%MEND mvn;

*VALUE LABELS FOR EACH LEVEL OF OUR CATEGORICAL VARIABLES;
PROC FORMAT;
VALUE exercises
    1 = "Aerobic Stair Climbing"
    2 = "Racquetball"
    3 = "Weight Training";

VALUE diets
    1 = "Meat Eaters"
    2 = "Vegetarians";

VALUE intensities
	1 = "Pulse: Warm Up"
	2 = "Pulse: Jogging"
	3 = "Pulse: Running";
RUN;


DATA work.dietwide;
INPUT exertype pulse1 pulse2 pulse3 diet personID;

*LABELING VARIABLES;
LABEL exertype = "Exercise Type"
      pulse1   = "Pulse After Warmup"
      pulse2   = "Pulse After Jogging"
      pulse3   = "Pulse After Running"
      diet     = "Diet Type";

*ADDING VALUE LABELS TO VARIABLES;
FORMAT exertype exercises. diet diets.;

*CREATING DUMMY CODED VARIABLES FOR EXERCISE TYPE:;
IF exertype = 1 THEN DO; dEXERCISE_ASC = 1; dEXERCISE_R = 0; dEXERCISE_WT = 0; END;
IF exertype = 2 THEN DO; dEXERCISE_ASC = 0; dEXERCISE_R = 1; dEXERCISE_WT = 0; END;
IF exertype = 3 THEN DO; dEXERCISE_ASC = 0; dEXERCISE_R = 0; dEXERCISE_WT = 1; END;


*CREATING DUMMY CODED VARIABLES FOR DIET TYPE;
IF diet = 1 THEN DO; dDIET_M = 1; dDIET_V = 0; END;
IF diet = 2 THEN DO; dDIET_M = 0; dDIET_V = 1; END;

*IMPORTING DATA;
DATALINES;
1         112       166       215       1      1
1         111       166       225       1      2
1         89        132       189       1      3
1         95        134       186       2      4
1         66        109       150       2      5
1         69        119       177       2      6
2         125       177       241       1      7
2         85        117       186       1      8
2         97        137       185       1      9
2         93        151       217       2     10
2         77        122       178       2     11
2         78        119       173       2     12
3         81        134       205       1     13
3         88        133       180       1     14
3         88        157       224       1     15
3         58         99       131       2     16
3         85        132       186       2     17
3         78        110       164       2     18
;
RUN;


*CONVERTING DATA TO STACKED FORM FOR PROC MIXED;
DATA WORK.dietstack;
    SET WORK.dietwide;

	FORMAT intensity intensities.; *ADDING A FORMAT STATEMENT FOR INTENSITY VARIABLE;

	*FIRST OUTCOME: PULSE 1 (AFTER WARM UP);
	pulse = pulse1;
    intensity = 1; 
	dINTENSITY_W = 1; dINTENSITY_J = 0; dINTENSITY_R = 0; *DUMMY CODED VARIABLES FOR ANALYSIS;
    OUTPUT; *OUTPUT MAKES THE LINE OF DATA GET WRITTEN TO THE NEW DATA SET;

	*SECOND OUTCOME: PULSE 2 (AFTER JOGGING);
    pulse = pulse2;
    intensity = 2; 
	dINTENSITY_W = 0; dINTENSITY_J = 1; dINTENSITY_R = 0; 
    OUTPUT;
	
	*THIRD OUTCOME: PULSE 3 (AFTER RUNNING);
    pulse = pulse3;
    intensity = 3; 
	dINTENSITY_W = 0; dINTENSITY_J = 0; dINTENSITY_R = 1;
    OUTPUT;

RUN;


*ORIGINAL ANALYSIS - ADDING LSMEANS STATEMENT FOR PREDICTED MEANS FOR EACH CONDITION;
TITLE "MULTIVARIATE MODEL WITH DIET PREDICTOR ONLY - CLASS STATEMENT";
PROC MIXED DATA=WORK.dietstack METHOD=ML COVTEST NOPROFILE ITDETAILS IC NAMELEN=50;
CLASS intensity diet exertype;
MODEL pulse = exertype|intensity|diet / S DDFM=KENWARDROGER;
REPEATED intensity / SUBJECT=personID TYPE=UN R RCORR;
LSMEANS exertype|intensity|diet;
RUN;
TITLE;
 
*FOR DATA GENERATION LATER;
DATA predmeans;
INPUT diet exertype pulse1p pulse2p pulse3p;
DATALINES;
1 1 104.00 154.67 209.67
1 2 102.33 143.67 204.00
1 3 85.6667 141.33 203.00
2 1 76.6667 120.67 171.00
2 2 82.6667 130.67 189.33
2 3 73.6667 113.67 160.33
;
RUN;

*STEP #1: GENERATE ERROR TERMS FROM MULTIVARIATE NORMAL DISTRIBUTION USING MULTNORM MACRO:;
DATA rmatrix;
INPUT col1-col3;
DATALINES;
126.41      162.87      185.76
162.87      250.67      290.13
185.76      290.13      395.22
;
RUN;

DATA meanvec;
INPUT col1;
DATALINES;
0
0
0
;
RUN;

*BEGIN DATA MAKING MACRO FOR REPLICATIONS;
DATA mvnall;
RUN;

%MACRO makedata(reps);

%DO i=1 %TO &reps.;
	%mvn(varcov=rmatrix,means=meanvec,n=18,seed=&i.,sample=mvnerror); 

	DATA mvnerror; SET mvnerror; 
		error_warmup = col1;
		error_jogging = col2;
		error_running = col3;
		personID = _N_;
		DROP col1-col3;
		replication = &i.;
	RUN;

	DATA mvnall;
		SET mvnall mvnerror;
		IF personID=. THEN DELETE;
	RUN;
%END;

%MEND;

%MAKEDATA(100);

*STEP 2: MERGE MVN ERROR TERMS WITH ORIGINAL DATA;
PROC SORT DATA=mvnall;
BY personID;
RUN;

DATA merge1;
	MERGE mvnall dietwide;
	BY personID;
	IF personID=. THEN DELETE;
RUN;

PROC SORT DATA=merge1;
BY exertype diet;
RUN;

PROC SORT DATA=predmeans;
BY exertype diet;
RUN;

DATA merge2;
	MERGE merge1 predmeans;
	BY exertype diet;
	pulse1 = pulse1p+error_warmup;
	pulse2 = pulse2p+error_jogging;
	pulse3 = pulse3p+error_running;

RUN;

*CONVERTING DATA TO STACKED FORM FOR PROC MIXED;
DATA stackreplications;
    SET merge2;

	*FIRST OUTCOME: PULSE 1 (AFTER WARM UP);
	pulse = pulse1;
    intensity = 1;
    OUTPUT; 

	*SECOND OUTCOME: PULSE 2 (AFTER JOGGING);
    pulse = pulse2;
    intensity = 2; 
    OUTPUT;
	
	*THIRD OUTCOME: PULSE 3 (AFTER RUNNING);
    pulse = pulse3;
    intensity = 3; 
    OUTPUT;

RUN;

PROC SORT DATA=stackreplications;
BY replication;
RUN;

*ORIGINAL ANALYSIS - ADDING LSMEANS STATEMENT FOR PREDICTED MEANS FOR EACH CONDITION;
TITLE "MULTIVARIATE MODEL WITH DIET PREDICTOR ONLY - CLASS STATEMENT";
PROC MIXED DATA=WORK.stackreplications METHOD=ML COVTEST NOPROFILE ITDETAILS IC NAMELEN=50;
BY replication;
CLASS intensity diet exertype;
MODEL pulse = exertype|intensity|diet / S DDFM=KENWARDROGER;
REPEATED intensity / SUBJECT=personID TYPE=UN R RCORR;
ODS OUTPUT Tests3 = Tests3;
RUN;
TITLE;

DATA Tests3; SET Tests3;
	IF ProbF LT .05 THEN Signif=1; ELSE Signif=0;
RUN;

PROC SORT DATA=Tests3; BY effect; RUN;
TITLE "Proportion Significant Omnibus F-test for Group";
PROC MEANS DATA=Tests3 MEAN; CLASS effect; VAR Signif; RUN;

